defmodule LinkShortener.Links do
  use Ecto.Schema
  import Ecto.Changeset
  import Ecto.Query
  alias LinkShortener.Repo
  alias LinkShortener.Links
  alias LinkShortener.Generators.SafeString

  schema "links" do
    field :name, :string
    field :url, :string
    field :shorten, :string

    timestamps()
  end

  @doc false
  def changeset(link, attrs) do
    link
    |> cast(attrs, [:name, :url, :shorten])
    |> validate_required([:url, :shorten])
    |> unique_constraint(:shorten)
  end

  def new_one(), do: Links.changeset(%Links{})

  def create_one(attrs, length \\ 10, generator \\ SafeString) do
    shorten = generator.generate(length)
    insert_one(Map.put(attrs, :shorten, shorten))
  end

  def insert_one(attrs) do
    %Links{}
    |> Links.changeset(attrs)
    |> Repo.insert()
  end

  def edit_one(id) do
    get_one(id)
    |> Links.changeset()
  end

  def update_one(%Links{} = Links, changes) do
    Links
    |> Links.changeset(changes)
    |> Repo.update()
  end

  def insert_one(attrs) do
    %Links{}
    |> Links.changeset(attrs)
    |> Repo.insert()
  end

  def delete_one(%Links{} = Links), do: Repo.delete(Links)

  def get_one_by(attrs) do
    Repo.get_by(Links, attrs)
  end

  def get_one_by_shorten(shorten) do
    get_one_by(%{shorten: shorten})
  end

  def get_one(id) do
    Repo.get!(Links, id)
  end

  def get_all(opts) do
    from(Links)
    |> Repo.all()
  end
end
